//===- ArrayRemap.h - Remaps Memory layout based on accesses for a single dimensional array ---===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//
//
// Copyright (c) 2015 Advanced Micro Devices, Inc.  All rights reserved.
// Developed by: Advanced Micro Devices, Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// with the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// Redistributions of source code must retain the above copyright notice, this
// list of conditions and the following disclaimers.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimers in the documentation
// and/or other materials provided with the distribution.
// Neither the names of Advanced Micro Devices, Inc., nor the names of its
// contributors may be used to endorse or promote products derived from this
// Software without specific prior written permission.
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH
// THE SOFTWARE.
//
//===----------------------------------------------------------------------===//
// ///
//   This Pass transforms the access of the form a[i*numGroups+Groupid] 
//   to a[GroupId*NumGroupElement+i]. This can reduce the number of cache misses across iterations.
//   
//   The passes looks at the malloc calls and all the accesses from the pointer of the malloc
//   calls. If all the accesses of this pointer in the program are of the form a[i*numGroups+GroupID], this
//   passes changes the GEPs of all the accesses to the form a[GroupID*numGroupElements+i]
//   
//   This is an LTO pass. This cannot work on Vector Type instructions. 
//   LoopVectorizer is disabled if -fremap-arrays is enabled in first pass compilation.
//   Vectorizer is only run in the LTO phase if -fremap-arrays is used in clang. 
//   
//   We use SCEVs to identify  the access patterns (GroupID and numGroups) of the arrays.
//   
//
//===----------------------------------------------------------------------===//

#ifndef LLVM_TRANSFORMS_IPO_ARRAYREMAP_H
#define LLVM_TRANSFORMS_IPO_ARRAYREMAP_H

#include "llvm/IR/PassManager.h"

namespace llvm {

  struct AROPass: public PassInfoMixin<AROPass> {
      PreservedAnalyses run(Module &M, ModuleAnalysisManager &AM);
  };
} // namespace llvm

#endif // LLVM_TRANSFORMS_IPO_ARRAYREMAP_H

