//===-- HandleMemFuncs.cpp - Prevents direct use of I8 ptr from Alloc routine ---===//
//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//
// This file prevents the direct use of byte ptr from malloc/calloc/realloc
// calls. This done by locking the Alloc calls at the start of complation
// and later unlocking them at the end of compilation phase. This helps to
// analyse and transform structures types.
//
//===----------------------------------------------------------------------===//
//====  Copyright (c) 2015 Advanced Micro Devices, Inc.  All rights reserved.
//
//               Developed by: Advanced Micro Devices, Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// with the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// Redistributions of source code must retain the above copyright notice, this
// list of conditions and the following disclaimers.
//
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimers in the documentation
// and/or other materials provided with the distribution.
//
// Neither the names of Advanced Micro Devices, Inc., nor the names of its
// contributors may be used to endorse or promote products derived from this
// Software without specific prior written permission.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH
// THE SOFTWARE.
//===----------------------------------------------------------------------===//

#ifndef LLVM_TRANSFORMS_IPO_HANDLE_MEM_FUNCS_H
#define LLVM_TRANSFORMS_IPO_HANDLE_MEM_FUNCS_H

#include "llvm/IR/PassManager.h"

namespace llvm {

    struct HandleMemFuncsPass : public PassInfoMixin<HandleMemFuncsPass> {
      int InvokeId;

    public:
        explicit HandleMemFuncsPass(int InvokeId) : InvokeId(InvokeId) {}

        PreservedAnalyses run(Module& M, ModuleAnalysisManager& AM);
    };
} // namespace llvm

#endif // LLVM_TRANSFORMS_IPO_HANDLE_MEM_FUNCS_H
