//===============           - ClassHierarchyUtils.h - ===============//
//
// Utility: This utility uses information obtained from Class Hierarchy analysis
// to check for class properties such as  only private members or class inherits
// only protected members etc. It also collects class methods that could
// potentially access the class's data members including methods of base classes
// which the class derives from.
//===----------------------------------------------------------------------===//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//
// Copyright (c) 2015 Advanced Micro Devices, Inc.  All rights reserved.
// Developed by: Advanced Micro Devices, Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// with the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// Redistributions of source code must retain the above copyright notice, this
// list of conditions and the following disclaimers.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimers in the documentation
// and/or other materials provided with the distribution.
// Neither the names of Advanced Micro Devices, Inc., nor the names of its
// contributors may be used to endorse or promote products derived from this
// Software without specific prior written permission.
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH
// THE SOFTWARE.
//
//===----------------------------------------------------------------------===//

#ifndef LLVM_TRANSFORMS_UTILS_CLASSHIERARCHYUTILS_H
#define LLVM_TRANSFORMS_UTILS_CLASSHIERARCHYUTILS_H
#include "llvm/Analysis/ClassHierarchyAnalysis.h"
#include "llvm/IR/BasicBlock.h"
#include "llvm/IR/Function.h"
#include "llvm/IR/Instructions.h"
#include "llvm/IR/Module.h"
#include "llvm/Pass.h"
#include "llvm/Passes/PassBuilder.h"
#include <map>

namespace llvm {
class ClassHierarchyUtils {
  std::map<llvm::GlobalVariable *, CXXClassInfo *> CXXClassInfoMap;
  SmallPtrSet<GlobalVariable *, 32> Visited;
  SmallPtrSet<CXXClassInfo *, 32> VisitedClasses;

public:
  std::vector<Function *> CollectedFunctions;
  std::vector<StringRef> CollectedFunctionNames;
  std::vector<llvm::GlobalVariable *> CollectedClasses;
  ClassHierarchyUtils(
      const std::map<llvm::GlobalVariable *, CXXClassInfo *> &CXXClassInfoMap)
      : CXXClassInfoMap(CXXClassInfoMap) {}
  bool checkClassHasPrivateMembersOnly(CXXClassInfo *ObjPtr);
  bool isaBaseClass(GlobalVariable *GV);
  int checkClassInheritsMembers(CXXClassInfo *ObjPtr);
  void collectFunctionsToAnalyze(GlobalVariable *GV, bool TraverseBase = true);
  void traverseCXXClassInfo(CXXClassInfo *Root, CXXClassInfo *Start);
  bool traverseCXXClassInfoCollectFns(GlobalVariable *GV,
                                      bool TraverseBase = true);
};
} // namespace llvm
#endif
