
//===- PreRvTransforms.cpp - Loop Canonicalization Pass ----------------------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//====  Copyright (c) 2015 Advanced Micro Devices, Inc.  All rights reserved.
//
//               Developed by: Advanced Micro Devices, Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// with the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// Redistributions of source code must retain the above copyright notice, this
// list of conditions and the following disclaimers.
//
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimers in the documentation
// and/or other materials provided with the distribution.
//
// Neither the names of Advanced Micro Devices, Inc., nor the names of its
// contributors may be used to endorse or promote products derived from this
// Software without specific prior written permission.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH
// THE SOFTWARE.
//
//===----------------------------------------------------------------------===//
//
//===----------------------------------------------------------------------===//
// This pass does some transformations that are required for enabling Region vectorization
// The transforms that are done in this pass are
// 1) Move loop invariant conditions like null pointer check to the preheader of the loop
// 2) If the loop invariant bounds are computed inside the loop, move them out of the loop
// to the preheader.
// 3) Convert load store reduction operations to loop invariant address location in presence of
//  of load/store operations to the same memory location  to Variable and do the load/Store
//  to the memory location in the loop exit block
//      e.g if the loop body has two instructions like
//         a[ constindex ] += Value1;
//         a[i] +=  Value2;
//         transform the loop body to
//
//          {
//         loopred += Value1 ;
//         a[i] += Value2;
//
//         }
//        loopexitBlock:
//           a[constindex] += loopred;
//
//
//===----------------------------------------------------------------------===//
#ifndef LLVM_TRANSFORMS_UTILS_RVTRANSFORMS_H
#define LLVM_TRANSFORMS_UTILS_RVTRANSFORMS_H

#include "llvm/IR/Function.h"
#include "llvm/IR/PassManager.h"

namespace llvm {

/// An optimization pass inserting data prefetches in loops.
class PreRVTransformsPass : public PassInfoMixin<PreRVTransformsPass> {
public:
  PreRVTransformsPass() = default;

  /// Run the pass over the function.
  PreservedAnalyses run(Function &F, FunctionAnalysisManager &AM);
};

} // end namespace llvm

#endif // LLVM_TRANSFORMS_UTILS_RVTRANSFORMS_H
