//===-------------------- SafetyUtil.h -----------------------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//====  Copyright (c) 2015 Advanced Micro Devices, Inc.  All rights reserved.
//
//               Developed by: Advanced Micro Devices, Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// with the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// Redistributions of source code must retain the above copyright notice, this
// list of conditions and the following disclaimers.
//
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimers in the documentation
// and/or other materials provided with the distribution.
//
// Neither the names of Advanced Micro Devices, Inc., nor the names of its
// contributors may be used to endorse or promote products derived from this
// Software without specific prior written permission.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH
// THE SOFTWARE.
//===----------------------------------------------------------------------===//

#ifndef LLVM_TRANSFORMS_SAFETY_UTILS_H
#define LLVM_TRANSFORMS_SAFETY_UTILS_H

#include "llvm/IR/Module.h"

namespace llvm {

// Function to check if there is any function with external linkage in module.
bool hasExternalLinkageForMELV(Module *M);

/// This function builds list of safe ilibrary functions which neither do not
/// modify heap memory nor do not call any other function indirectly
///
/// TODO: This list must be updated as and when new safe library call is
/// discovered
void buildSafeLibFuncs();

// This function checks the linkage of all the functions to avoid if any
// external library calls using the vectorized code on unpadded memory. We
// ensure that none of the functions are being called from external library
// functions. This function returns false if any function other than main has
// external linkage. All the library calls are looked up in a table of safe
// functions if a library call is not safe then also returns false.
bool checkLinkage(Module *M);

} // end namespace llvm

#endif
