//===- SparseArrayUtil.h - sparse array help transforms ---==============//
//* Some Legal Stuff
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//====  Copyright (c) 2015 Advanced Micro Devices, Inc.  All rights reserved.
//
//               Developed by: Advanced Micro Devices, Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// with the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// Redistributions of source code must retain the above copyright notice, this
// list of conditions and the following disclaimers.
//
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimers in the documentation
// and/or other materials provided with the distribution.
//
// Neither the names of Advanced Micro Devices, Inc., nor the names of its
// contributors may be used to endorse or promote products derived from this
// Software without specific prior written permission.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH
// THE SOFTWARE.
//===----------------------------------------------------------------------===//
#ifndef LLVM_TRANSFORMS_UTILS_SPARSEARRAYUTIL_H
#define LLVM_TRANSFORMS_UTILS_SPARSEARRAYUTIL_H

#include "llvm/IR/PassManager.h"

namespace llvm {
namespace aocc {
using namespace llvm;

class SparseArraySetupPass
  :public PassInfoMixin<SparseArraySetupPass> {
  bool Setup;
  bool OnlyLoops;
public:
  SparseArraySetupPass(bool Setup, bool OnlyLoops) :Setup(Setup), OnlyLoops(OnlyLoops) { }
  PreservedAnalyses run(Function &F, FunctionAnalysisManager& M);
};

class EliminateLoopForSingletonListPass
  :public PassInfoMixin<EliminateLoopForSingletonListPass> {
public:
  EliminateLoopForSingletonListPass() { }
  PreservedAnalyses run(Function &F, FunctionAnalysisManager& M);
};

struct SparseArrayPropagationAction
{
  SparsificationPass *SP;
  LoopInfo& LI;
  AliasAnalysis *AA;
  TargetLibraryInfo& TLI;
  DominatorTree& DT;
  SparseArrayPropagationAction(SparsificationPass *SP, LoopInfo& LI, AliasAnalysis *AA, TargetLibraryInfo& TLI, DominatorTree& DT)
    :SP(SP), LI(LI), AA(AA), TLI(TLI), DT(DT) { }
  bool transform(Function& );
  DenseMap<LoadInst*, SmallVector<Value*, 2>> SparseStoreAliasMap;
  private:
  DenseMap<std::pair<Value*, Value*>, bool> SparseReachCache;
  bool isLoadFromSparseArray(Value *LI, Value *V,
                                  std::set<Value *> &Visited);
  bool isLoadFromSparseArray(Value *LI, Value *V);
};

class SparseArrayPropagationPass
  :public PassInfoMixin<SparseArrayPropagationPass> {
public:
  SparseArrayPropagationPass() { }
  PreservedAnalyses run(Function &F, FunctionAnalysisManager& M);
};

class RemoveRedundantLoadPass
  :public PassInfoMixin<RemoveRedundantLoadPass> {
public:
  RemoveRedundantLoadPass() { }
  PreservedAnalyses run(Function &F, FunctionAnalysisManager& M);
};

class EliminateLoopPreCondPass
  :public PassInfoMixin<EliminateLoopPreCondPass> {
public:
  EliminateLoopPreCondPass() { }
  PreservedAnalyses run(Function &F, FunctionAnalysisManager& M);
};

} // namespace aocc
} // namespace llvm

#endif
