##===- DetectX86CpuFeature.cmake -------------------------------------------===##
#
# Performs a try_run to check cpu feature flag of host
#
##===----------------------------------------------------------------------===##

function(detect_x86_feature feature variable)
    try_run(HAVE_RUN_CPU_FEATURE_${variable} HAVE_COMPILE_CPU_FEATURE_${variable}
            ${CMAKE_BINARY_DIR}
            ${CMAKE_SOURCE_DIR}/cmake/modules/DetectX86FeatureFlag.c
            COMPILE_DEFINITIONS -DFEATURE_FLAG=${feature}
            COMPILE_OUTPUT_VARIABLE COMP_OUTPUT
            RUN_OUTPUT_VARIABLE RUN_OUTPUT)

    if(HAVE_COMPILE_CPU_FEATURE_${variable} AND NOT (HAVE_RUN_CPU_FEATURE_${variable} STREQUAL  FAILED_TO_RUN))
        if(RUN_OUTPUT MATCHES "^(YES|NO)$")
            message(STATUS "Check host cpu feature ${feature}: ${RUN_OUTPUT}")
            set(${variable} ${RUN_OUTPUT} PARENT_SCOPE)
        else()
            message(SEND_ERROR "Could not detect host cpu feature ${feature}!")
            file(APPEND ${CMAKE_BINARY_DIR}${CMAKE_FILES_DIRECTORY}/CMakeError.log
                    "Determining the system cpu feature failed with the following output:\n${COMP_OUTPUT}\n${RUN_OUTPUT}")
        endif()
    else()
        message(STATUS "Check host system for feature ${feature} - failed")
        file(APPEND ${CMAKE_BINARY_DIR}${CMAKE_FILES_DIRECTORY}/CMakeError.log
                "Determining the system cpu feature failed with the following output:\n${COMP_OUTPUT}\n${RUN_OUTPUT}")
        set(${variable} PARENT_SCOPE)
    endif()
endfunction(detect_x86_feature)